"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.displayFailures = exports.displayTestResults = exports.displaySuccesses = exports.toArray = exports.sortTestResults = exports.sortFileResponses = exports.asRelativePaths = void 0;
const os = require("os");
const path = require("path");
const cli_ux_1 = require("cli-ux");
const chalk_1 = require("chalk");
const sdr_1 = require("@sf/sdr");
const types_1 = require("@sf/sdr/lib/src/client/types");
const ts_types_1 = require("@salesforce/ts-types");
const testLevel_1 = require("./testLevel");
function info(message) {
    return chalk_1.blue(chalk_1.bold(message));
}
function error(message) {
    return chalk_1.red(chalk_1.bold(message));
}
function asRelativePaths(fileResponses) {
    fileResponses.forEach((file) => {
        if (file.filePath) {
            file.filePath = path.relative(process.cwd(), file.filePath);
        }
    });
    return fileResponses;
}
exports.asRelativePaths = asRelativePaths;
/**
 * Sorts file responds by type, then by filePath, then by fullName
 */
function sortFileResponses(fileResponses) {
    return fileResponses.sort((i, j) => {
        if (i.type === j.type && i.filePath && j.filePath) {
            if (i.filePath === j.filePath) {
                return i.fullName > j.fullName ? 1 : -1;
            }
            return (i === null || i === void 0 ? void 0 : i.filePath) > (j === null || j === void 0 ? void 0 : j.filePath) ? 1 : -1;
        }
        return i.type > j.type ? 1 : -1;
    });
}
exports.sortFileResponses = sortFileResponses;
function sortTestResults(results = []) {
    return results.sort((a, b) => {
        if (a.methodName === b.methodName) {
            return a.name.localeCompare(b.name);
        }
        return a.methodName.localeCompare(b.methodName);
    });
}
exports.sortTestResults = sortTestResults;
function toArray(entryOrArray) {
    if (entryOrArray) {
        return Array.isArray(entryOrArray) ? entryOrArray : [entryOrArray];
    }
    return [];
}
exports.toArray = toArray;
function displaySuccesses(result) {
    var _a;
    const fileResponses = asRelativePaths((_a = result.getFileResponses()) !== null && _a !== void 0 ? _a : []);
    const successes = sortFileResponses(fileResponses.filter((f) => f.state !== 'Failed'));
    if (!successes.length)
        return;
    const columns = {
        state: { header: 'State' },
        fullName: { header: 'Name' },
        type: { header: 'Type' },
        filePath: { header: 'Path' },
    };
    const title = result instanceof sdr_1.DeployResult ? 'Deployed Source' : 'Retrieved Source';
    const options = { title: info(title) };
    cli_ux_1.default.log();
    cli_ux_1.default.table(successes, columns, options);
}
exports.displaySuccesses = displaySuccesses;
function displayTestResults(result, testLevel) {
    var _a, _b, _c, _d;
    if (testLevel === testLevel_1.TestLevel.NoTestRun) {
        cli_ux_1.default.log();
        return;
    }
    if ((_a = result === null || result === void 0 ? void 0 : result.response) === null || _a === void 0 ? void 0 : _a.numberTestErrors) {
        const failures = toArray((_c = (_b = result.response.details) === null || _b === void 0 ? void 0 : _b.runTestResult) === null || _c === void 0 ? void 0 : _c.failures);
        const failureCount = (_d = result.response.details.runTestResult) === null || _d === void 0 ? void 0 : _d.numFailures;
        const tests = sortTestResults(failures);
        cli_ux_1.default.log();
        cli_ux_1.default.log(error(`Test Failures [${failureCount}]`));
        for (const test of tests) {
            const testName = chalk_1.underline(`${test.name}.${test.methodName}`);
            const stackTrace = test.stackTrace.replace(/\n/g, `${os.EOL}    `);
            cli_ux_1.default.log(`• ${testName}`);
            cli_ux_1.default.log(`  ${chalk_1.dim('message')}: ${test.message}`);
            cli_ux_1.default.log(`  ${chalk_1.dim('stacktrace')}: ${os.EOL}    ${stackTrace}`);
            cli_ux_1.default.log();
        }
    }
    cli_ux_1.default.log();
    cli_ux_1.default.log(info('Test Results Summary'));
    const passing = ts_types_1.get(result, 'response.numberTestsCompleted', 0);
    const failing = ts_types_1.get(result, 'response.numberTestErrors', 0);
    const total = ts_types_1.get(result, 'response.numberTestsTotal', 0);
    const time = ts_types_1.get(result, 'response.details.runTestResult.totalTime', 0);
    cli_ux_1.default.log(`Passing: ${passing}`);
    cli_ux_1.default.log(`Failing: ${failing}`);
    cli_ux_1.default.log(`Total: ${total}`);
    if (time)
        cli_ux_1.default.log(`Time: ${time}`);
}
exports.displayTestResults = displayTestResults;
function displayFailures(result) {
    var _a;
    if (result.response.status === types_1.RequestStatus.Succeeded)
        return;
    const fileResponses = asRelativePaths((_a = result.getFileResponses()) !== null && _a !== void 0 ? _a : []);
    const failures = sortFileResponses(fileResponses.filter((f) => f.state === 'Failed'));
    if (!failures.length)
        return;
    const columns = {
        problemType: { header: 'Type' },
        fullName: { header: 'Name' },
        error: { header: 'Problem' },
    };
    const options = { title: error(`Component Failures [${failures.length}]`) };
    cli_ux_1.default.log();
    cli_ux_1.default.table(failures, columns, options);
}
exports.displayFailures = displayFailures;
//# sourceMappingURL=output.js.map